@@ -1,3 +1,9 @@
+# Copyright (c) 2010 Riverbank Computing Limited.
+# Copyright (c) 2006 Torsten Marek.
+
+
+import os.path
+
 from PyQt4 import QtGui, QtCore
 from PyQt4.uic.uiparser import UIParser
 from PyQt4.uic.Loader.qobjectcreator import LoaderCreatorPolicy
@@ -16,6 +22,10 @@ class DynamicUILoader(UIParser):
         else:
             return self.factory.createQObject(classname, widgetname, ())
 
-    def loadUi(self, filename, toplevelInst = None):
+    def loadUi(self, filename, toplevelInst=None):
         self.toplevelInst = toplevelInst
-        return self.parse(filename)
+
+        # Allow the filename to be a QString.
+        filename = str(filename)
+
+        return self.parse(filename, os.path.dirname(filename))
