@@ -52,14 +52,31 @@ QHash<QByteArray, QList<const Chimera *> > Chimera::_previously_parsed;
 
 // Construct an invalid type.
 Chimera::Chimera()
-    : _type(0), _metatype(QMetaType::Void), _inexact(false), _is_flag(false)
+    : _type(0), _py_type(0), _metatype(QMetaType::Void), _inexact(false),
+      _is_flag(false)
 {
 }
 
 
+// Construct a copy.
+Chimera::Chimera(const Chimera &other)
+{
+    _type = other._type;
+
+    _py_type = other._py_type;
+    Py_XINCREF(_py_type);
+    
+    _metatype = other._metatype;
+    _inexact = other._inexact;
+    _is_flag = other._is_flag;
+    _name = other._name;
+}
+
+
 // Destroy the type.
 Chimera::~Chimera()
 {
+    Py_XDECREF(_py_type);
 }
 
 
@@ -162,25 +179,63 @@ Chimera::Signature *Chimera::parse(const QByteArray &sig, const char *context)
 
         if (parsed_args.isEmpty())
         {
-            QList<QByteArray> args = args_str.split(',');
+            int i, arg_start, template_level;
 
-            for (QList<QByteArray>::const_iterator it = args.constBegin(); it != args.constEnd(); ++it)
+            i = arg_start = template_level = 0;
+
+            // Extract each argument allowing for commas in templates.
+            for (;;)
             {
-                Chimera *ct = new Chimera;
+                char ch = (i < args_str.size() ? args_str.at(i) : '\0');
+                QByteArray arg;
 
-                if (!ct->parse_cpp_type(*it))
+                switch (ch)
                 {
-                    delete ct;
-                    delete parsed_sig;
-                    qDeleteAll(parsed_args.constBegin(),
-                            parsed_args.constEnd());
+                case '<':
+                    ++template_level;
+                    break;
 
-                    raiseParseException(it->constData(), context);
+                case '>':
+                    --template_level;
+                    break;
 
-                    return 0;
+                case '\0':
+                    arg = args_str.mid(arg_start, i - arg_start);
+                    break;
+
+                case ',':
+                    if (template_level == 0)
+                    {
+                        arg = args_str.mid(arg_start, i - arg_start);
+                        arg_start = i + 1;
+                    }
+
+                    break;
                 }
 
-                parsed_args.append(ct);
+                if (!arg.isEmpty())
+                {
+                    Chimera *ct = new Chimera;
+
+                    if (!ct->parse_cpp_type(arg))
+                    {
+                        delete ct;
+                        delete parsed_sig;
+                        qDeleteAll(parsed_args.constBegin(),
+                                parsed_args.constEnd());
+
+                        raiseParseException(arg.constData(), context);
+
+                        return 0;
+                    }
+
+                    parsed_args.append(ct);
+
+                    if (ch == '\0')
+                        break;
+                }
+
+                ++i;
             }
 
             // Only parse once.
@@ -205,6 +260,7 @@ Chimera::Signature *Chimera::parse(PyObject *types, const char *name,
     Chimera::Signature *parsed_sig = new Chimera::Signature(name, false);
 
     parsed_sig->signature.append('(');
+    parsed_sig->py_signature.append('[');
 
     for (SIP_SSIZE_T i = 0; i < PyTuple_GET_SIZE(types); ++i)
     {
@@ -223,28 +279,52 @@ Chimera::Signature *Chimera::parse(PyObject *types, const char *name,
         parsed_sig->parsed_arguments.append(parsed_type);
 
         if (i > 0)
+        {
             parsed_sig->signature.append(',');
+            parsed_sig->py_signature.append(", ");
+        }
 
         parsed_sig->signature.append(parsed_type->name());
+
+        if (parsed_type->_py_type)
+            parsed_sig->py_signature.append(((PyTypeObject *)parsed_type->_py_type)->tp_name);
+        else
+            parsed_sig->py_signature.append(parsed_type->name());
     }
 
     parsed_sig->signature.append(')');
+    parsed_sig->py_signature.append(']');
 
     return parsed_sig;
 }
 
 
-// Raise an exception after parse() has failed.
-void Chimera::raiseParseException(PyObject *obj, const char *context)
+// Raise an exception after parse() of a Python type has failed.
+void Chimera::raiseParseException(PyObject *type, const char *context)
 {
-    raiseParseException(Py_TYPE(obj)->tp_name, context);
+    if (PyType_Check(type))
+    {
+        PyErr_Format(PyExc_TypeError,
+                "Python type '%s' is not supported as %s type",
+                ((PyTypeObject *)type)->tp_name, context);
+    }
+    else
+    {
+        const char *cpp_type_name = sipString_AsASCIIString(&type);
+
+        if (cpp_type_name)
+        {
+            raiseParseException(cpp_type_name, context);
+            Py_DECREF(type);
+        }
+    }
 }
 
 
-// Raise an exception after parse() has failed.
+// Raise an exception after parse() of a C++ type has failed.
 void Chimera::raiseParseException(const char *type, const char *context)
 {
-    PyErr_Format(PyExc_TypeError, "type '%s' is not supported as %s type",
+    PyErr_Format(PyExc_TypeError, "C++ type '%s' is not supported as %s type",
             type, context);
 }
 
@@ -252,6 +332,9 @@ void Chimera::raiseParseException(const char *type, const char *context)
 // Parse the given Python type object.
 bool Chimera::parse_py_type(PyTypeObject *type_obj)
 {
+    _py_type = (PyObject *)type_obj;
+    Py_INCREF(_py_type);
+
     const sipTypeDef *td = sipTypeFromPyTypeObject(type_obj);
 
     if (td)
@@ -346,16 +429,6 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
     {
         _metatype = QMetaType::Double;
     }
-    else if (type_obj == &PyList_Type)
-    {
-        _metatype = QMetaType::QVariantList;
-    }
-    else if (type_obj == &PyDict_Type)
-    {
-        // Strictly speaking a QVariantHash is a closer match to a Python dict,
-        // but that is only Qt v4.5 and later.
-        _metatype = QMetaType::QVariantMap;
-    }
 
     // Fallback to using a PyQt_PyObject.
     if (_metatype == QMetaType::Void)
@@ -377,54 +450,182 @@ void Chimera::set_flag()
 }
 
 
+// Update a C++ type so that any typedefs and registered ints are resolved.
+QByteArray Chimera::resolve_types(const QByteArray &type)
+{
+    // Split into a base type and a possible list of template arguments.
+    QByteArray resolved = type.simplified();
+
+    // Get the raw type, ie. without any "const", "&" or "*".
+    QByteArray raw_type;
+    int original_raw_start;
+
+    if (resolved.startsWith("const "))
+        original_raw_start = 6;
+    else
+        original_raw_start = 0;
+
+    raw_type = resolved.mid(original_raw_start);
+
+    while (raw_type.endsWith('&') || raw_type.endsWith('*') || raw_type.endsWith(' '))
+        raw_type.chop(1);
+
+    int original_raw_len = raw_type.size();
+
+    if (original_raw_len == 0)
+        return QByteArray();
+
+    // Get any template arguments.
+    QList<QByteArray> args;
+    int tstart = raw_type.indexOf('<');
+
+    if (tstart >= 0)
+    {
+        // Make sure the template arguments are terminated.
+        if (!raw_type.endsWith('>'))
+            return QByteArray();
+
+        // Split the template arguments taking nested templates into account.
+        int depth = 1, arg_start = tstart + 1;
+
+        for (int i = arg_start; i < raw_type.size(); ++i)
+        {
+            int arg_end = -1;
+            char ch = raw_type.at(i);
+
+            if (ch == '<')
+            {
+                ++depth;
+            }
+            else if (ch == '>')
+            {
+                --depth;
+
+                if (depth < 0)
+                    return QByteArray();
+
+                if (depth == 0)
+                    arg_end = i;
+            }
+            else if (ch == ',' && depth == 1)
+            {
+                arg_end = i;
+            }
+
+            if (arg_end >= 0)
+            {
+                QByteArray arg = resolve_types(raw_type.mid(arg_start, arg_end - arg_start));
+
+                if (arg.isEmpty())
+                    return QByteArray();
+
+                args.append(arg);
+
+                arg_start = arg_end + 1;
+            }
+        }
+
+        if (depth != 0)
+            return QByteArray();
+
+        // Remove the template arguments.
+        raw_type.truncate(tstart);
+    }
+
+    // Expand any typedef.
+    const char *base_type = sipResolveTypedef(raw_type.constData());
+
+    if (base_type)
+        raw_type = base_type;
+
+    // Do the same for any registered int types.
+    if (_registered_int_types.contains(raw_type))
+        raw_type = "int";
+
+    // Add any (now resolved) template arguments.
+    if (args.count() > 0)
+    {
+        raw_type.append('<');
+
+        for (QList<QByteArray>::const_iterator it = args.begin();;)
+        {
+            raw_type.append(*it);
+
+            if (++it == args.end())
+                break;
+
+            raw_type.append(',');
+        }
+
+        if (raw_type.endsWith('>'))
+            raw_type.append(' ');
+
+        raw_type.append('>');
+    }
+
+    // Replace the original raw type with the resolved one.
+    resolved.replace(original_raw_start, original_raw_len, raw_type);
+
+    return resolved;
+}
+
+
 // Parse the given C++ type name.
 bool Chimera::parse_cpp_type(const QByteArray &type)
 {
-    // Get the expanded, normalised name.
-    QByteArray norm_name = _name = type;
+    _name = type;
 
-    // Extract the raw type and resolve any typedef.
-    QByteArray raw_type;
-    int raw_start;
-    const char *base_type;
+    // Resolve any types.
+    QByteArray resolved = resolve_types(type);
 
-    raw_start = extract_raw_type(norm_name, raw_type);
-    base_type = sipResolveTypedef(raw_type.constData());
-
-    if (base_type)
-        norm_name.replace(raw_start, raw_type.size(), base_type);
-
-    // Do the same for any registered int types.
-    if (!_registered_int_types.isEmpty())
-    {
-        raw_start = extract_raw_type(norm_name, raw_type);
-
-        if (_registered_int_types.contains(raw_type))
-            norm_name.replace(raw_start, raw_type.size(), "int");
-    }
+    if (resolved.isEmpty())
+        return false;
 
     // See if the type is known to Qt.
-    _metatype = QMetaType::type(norm_name.constData());
+    _metatype = QMetaType::type(resolved.constData());
 
     // If not then use the PyQt_PyObject wrapper.
     if (_metatype == QMetaType::Void)
         _metatype = PyQt_PyObject::metatype;
 
     // See if the type (without a pointer) is known to SIP.
-    bool is_ptr = norm_name.endsWith('*');
+    bool is_ptr = resolved.endsWith('*');
 
     if (is_ptr)
     {
-        norm_name.chop(1);
+        resolved.chop(1);
 
-        if (norm_name.endsWith('*'))
-            return true;
+        if (resolved.endsWith('*'))
+            return false;
     }
 
-    _type = sipFindType(norm_name.constData());
+    _type = sipFindType(resolved.constData());
 
     if (!_type)
-        return true;
+    {
+        // This is the only fundamental pointer type recognised by Qt.
+        if (_metatype == QMetaType::VoidStar)
+            return true;
+
+        // This is 'int', 'bool', etc.
+        if (_metatype != PyQt_PyObject::metatype && !is_ptr)
+            return true;
+
+        if ((resolved == "char" || resolved == "const char") && is_ptr)
+        {
+            // This is a special value meaning a (hopefully) '\0' terminated
+            // string.
+            _metatype = -1;
+
+            return true;
+        }
+
+        // This is an explicit 'PyQt_PyObject'.
+        if (resolved == "PyQt_PyObject" && !is_ptr)
+            return true;
+
+        return false;
+    }
 
     if (sipTypeIsNamespace(_type))
         return false;
@@ -459,37 +660,6 @@ bool Chimera::parse_cpp_type(const QByteArray &type)
 }
 
 
-// Extract the raw type from a normalised type name and return the start
-// position of the raw type in the original.
-int Chimera::extract_raw_type(const QByteArray &type, QByteArray &raw_type)
-{
-    // Find the start of the raw type.
-    int raw_start;
-
-    if (type.startsWith("const "))
-        raw_start = 6;
-    else
-        raw_start = 0;
-
-    // Find the length of the raw type, ie. before the start of any trailing
-    // indirection or reference.
-    int raw_len;
-
-    for (raw_len = type.size() - raw_start; raw_len >= 0; --raw_len)
-    {
-        char ch = type.at(raw_start + raw_len - 1);
-
-        if (ch != '&' && ch != '*')
-            break;
-    }
-
-    // Extract the raw type.
-    raw_type = type.mid(raw_start, raw_len);
-
-    return raw_start;
-}
-
-
 // Convert a Python object to C++ at a given address.  This has a lot in common
 // with the method that converts to a QVariant.  However, unlike that method,
 // we have no control over the size of the destination storage and so must
@@ -591,12 +761,21 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
 
     case QMetaType::QVariantList:
         {
-            QVariantList ql;
+            QList<QObject *> qlo;
 
-            if (to_QVariantList(py, ql))
-                *reinterpret_cast<QVariantList *>(cpp) = ql;
+            if (to_QList_QObject(py, qlo))
+            {
+                *reinterpret_cast<QList<QObject *> *>(cpp) = qlo;
+            }
             else
-                iserr = 1;
+            {
+                QVariantList ql;
+
+                if (to_QVariantList(py, ql))
+                    *reinterpret_cast<QVariantList *>(cpp) = ql;
+                else
+                    iserr = 1;
+            }
 
             break;
         }
@@ -627,6 +806,20 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
         }
 #endif
 
+    case -1:
+        {
+            char **ptr = reinterpret_cast<char **>(cpp);
+
+            if (SIPBytes_Check(py))
+                *ptr = SIPBytes_AS_STRING(py);
+            else if (py == Py_None)
+                *ptr = 0;
+            else
+                iserr = 1;
+
+            break;
+        }
+
     default:
         if (_type)
         {
@@ -695,12 +888,17 @@ sipAssignFunc Chimera::get_assign_helper() const
 
 
 // Convert a Python object to a QVariant.
-bool Chimera::fromPyObject(PyObject *py, QVariant *var) const
+bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
 {
     // Deal with the simple case of wrapping the Python object rather than
     // converting it.
     if (_type != sipType_QVariant && _metatype == PyQt_PyObject::metatype)
     {
+        // If the type was specified by a Python type (as opposed to
+        // 'PyQt_PyObject') then check the object is an instance of it.
+        if (_py_type && !PyObject_IsInstance(py, _py_type))
+            return false;
+
         *var = keep_as_pyobject(py);
         return true;
     }
@@ -875,16 +1073,26 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var) const
 
     case QMetaType::QVariantList:
         {
-            QVariantList ql;
+            QList<QObject *> qlo;
 
-            if (to_QVariantList(py, ql))
+            if (to_QList_QObject(py, qlo))
             {
-                *var = QVariant(ql);
+                *var = QVariant(QList_QObject_metatype(), &qlo);
                 metatype_used = QMetaType::Void;
             }
             else
             {
-                iserr = 1;
+                QVariantList ql;
+
+                if (to_QVariantList(py, ql))
+                {
+                    *var = QVariant(ql);
+                    metatype_used = QMetaType::Void;
+                }
+                else
+                {
+                    iserr = 1;
+                }
             }
 
             break;
@@ -899,6 +1107,10 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var) const
                 *var = QVariant(qm);
                 metatype_used = QMetaType::Void;
             }
+            else if (strict)
+            {
+                iserr = 1;
+            }
             else
             {
                 // Assume the failure is because the key was the wrong type.
@@ -923,17 +1135,25 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var) const
             }
             else
             {
-                // Assume the failure is because the key was the wrong type.
-                PyErr_Clear();
-
-                *var = keep_as_pyobject(py);
-                metatype_used = QMetaType::Void;
+                iserr = 1;
             }
 
             break;
         }
 #endif
 
+    case -1:
+        metatype_used = QMetaType::VoidStar;
+
+        if (SIPBytes_Check(py))
+            tmp_storage.tmp_void_ptr = SIPBytes_AS_STRING(py);
+        else if (py == Py_None)
+            tmp_storage.tmp_void_ptr = 0;
+        else
+            iserr = 1;
+
+        break;
+
     default:
         if (_type)
         {
@@ -994,8 +1214,26 @@ QVariant Chimera::fromAnyPyObject(PyObject *py, int *is_err)
     {
         Chimera ct;
 
-        if (!ct.parse_py_type(Py_TYPE(py)) || !ct.fromPyObject(py, &variant))
+        if (ct.parse_py_type(Py_TYPE(py)))
+        {
+            // If the type is a list or dict then try and convert it to a
+            // QVariantList or QVariantMap respectively if possible.
+            if (Py_TYPE(py) == &PyList_Type)
+                ct._metatype = QMetaType::QVariantList;
+            else if (Py_TYPE(py) == &PyDict_Type)
+                ct._metatype = QMetaType::QVariantMap;
+
+            // The conversion is non-strict in case the type was a dict and we
+            // can't convert it to a QVariantMap.
+            if (!ct.fromPyObject(py, &variant, false))
+            {
+                *is_err = 1;
+            }
+        }
+        else
+        {
             *is_err = 1;
+        }
     }
 
     return variant;
@@ -1037,6 +1275,34 @@ PyObject *Chimera::toPyObject(const QVariant &var) const
         }
     }
 
+    // Handle QList<QObject *>.
+    int lo_metatype = QList_QObject_metatype();
+
+    if (lo_metatype != 0 && lo_metatype == var.userType())
+    {
+        const QList<QObject *> *cpp = reinterpret_cast<const QList<QObject *> *>(var.data());
+
+        PyObject *py_list = PyList_New(cpp->count());
+
+        if (!py_list)
+            return 0;
+
+        for (int i = 0; i < cpp->count(); ++i)
+        {
+            PyObject *obj = sipConvertFromType(cpp->at(i), sipType_QObject, 0);
+
+            if (!obj)
+            {
+                Py_DECREF(py_list);
+                return 0;
+            }
+
+            PyList_SET_ITEM(py_list, i, obj);
+        }
+
+        return py_list;
+    }
+
     return toPyObject(const_cast<void *>(var.data()));
 }
 
@@ -1226,6 +1492,23 @@ PyObject *Chimera::toPyObject(void *cpp) const
         }
 #endif
 
+    case -1:
+        {
+            char *s = *reinterpret_cast<char **>(cpp);
+
+            if (s)
+            {
+                py = SIPBytes_FromString(s);
+            }
+            else
+            {
+                Py_INCREF(Py_None);
+                py = Py_None;
+            }
+
+            break;
+        }
+
     default:
         if (_type)
         {
@@ -1321,6 +1604,52 @@ QVariant Chimera::keep_as_pyobject(PyObject *py)
 }
 
 
+// Get the metatype for QList<QObject *> or 0 if it hasn't been registered.
+int Chimera::QList_QObject_metatype()
+{
+    static int lo_metatype = 0;
+
+    // We look each time until we find it because QtDeclarative could register
+    // it at any time.
+    if (lo_metatype == 0)
+        lo_metatype = QMetaType::type("QList<QObject*>");
+
+    return lo_metatype;
+}
+
+
+// Convert a Python list object to a QList<QObject*> and return true if there
+// was no error.
+bool Chimera::to_QList_QObject(PyObject *py, QList<QObject *>&cpp) const
+{
+    // We only try to convert if QList<QObject *> is a registered metatype.
+    if (QList_QObject_metatype() == 0)
+        return false;
+
+    Q_ASSERT(PyList_CheckExact(py));
+
+    for (SIP_SSIZE_T i = 0; i < PyList_GET_SIZE(py); ++i)
+    {
+        PyObject *val_obj = PyList_GET_ITEM(py, i);
+
+        if (!val_obj)
+            return false;
+
+        int iserr = 0;
+
+        QObject *val = reinterpret_cast<QObject *>(sipForceConvertToType(
+                val_obj, sipType_QObject, 0, SIP_NO_CONVERTORS, 0, &iserr));
+
+        if (iserr)
+            return false;
+
+        cpp.append(val);
+    }
+
+    return true;
+}
+
+
 // Convert a Python list object to a QVariantList and return true if there was
 // no error.
 bool Chimera::to_QVariantList(PyObject *py, QVariantList &cpp) const
@@ -1337,7 +1666,7 @@ bool Chimera::to_QVariantList(PyObject *py, QVariantList &cpp) const
         int val_state, iserr = 0;
 
         QVariant *val = reinterpret_cast<QVariant *>(sipForceConvertToType(
-                val_obj, sipType_QVariant, NULL, SIP_NOT_NONE, &val_state,
+                val_obj, sipType_QVariant, 0, SIP_NOT_NONE, &val_state,
                 &iserr));
 
         if (iserr)
