@@ -38,6 +38,7 @@
 #include "qpycore_misc.h"
 #include "qpycore_pyqtboundsignal.h"
 #include "qpycore_pyqtproxy.h"
+#include "qpycore_pyqtpyobject.h"
 #include "qpycore_pyqtsignal.h"
 #include "qpycore_sip.h"
 #include "qpycore_sip_helpers.h"
@@ -64,6 +65,12 @@ static PyObject *disconnect(qpycore_pyqtBoundSignal *bs, QObject *qrx,
         const char *slot);
 static QObject *get_receiver(Chimera::Signature *overload, PyObject *slot_obj,
         QByteArray &name);
+static QByteArray slot_signature_from_decorations(Chimera::Signature *signal,
+        PyObject *decorations, int nr_args);
+static QByteArray slot_signature_from_metaobject(Chimera::Signature *signal,
+        const QMetaObject *slot_mo, const QByteArray &slot_name, int nr_args);
+static QByteArray slot_signature(Chimera::Signature *signal,
+        const QByteArray &slot_name, int nr_args);
 
 
 // Doc-strings.
@@ -181,9 +188,8 @@ static PyObject *pyqtBoundSignal_get_doc(PyObject *self, void *)
 static PyObject *pyqtBoundSignal_repr(PyObject *self)
 {
     qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
-    qpycore_pyqtSignal *ps = (qpycore_pyqtSignal *)bs->unbound_signal;
 
-    QByteArray name = Chimera::Signature::name(ps->overloads->first()->signature);
+    QByteArray name = Chimera::Signature::name(bs->bound_overload->signature);
 
     return
 #if PY_MAJOR_VERSION >= 3
@@ -220,8 +226,7 @@ static void pyqtBoundSignal_dealloc(PyObject *self)
 
 // Create a bound signal.
 PyObject *qpycore_pyqtBoundSignal_New(PyObject *unbound_signal,
-        PyObject *bound_pyobject, QObject *bound_qobject,
-        Chimera::Signature *bound_overload)
+        PyObject *bound_pyobject, QObject *bound_qobject, int signal_index)
 {
     qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)PyType_GenericAlloc(&qpycore_pyqtBoundSignal_Type, 0);
 
@@ -232,7 +237,7 @@ PyObject *qpycore_pyqtBoundSignal_New(PyObject *unbound_signal,
 
         bs->bound_pyobject = bound_pyobject;
         bs->bound_qobject = bound_qobject;
-        bs->bound_overload = bound_overload;
+        bs->bound_overload = ((qpycore_pyqtSignal *)unbound_signal)->overloads->at(signal_index);
     }
 
     return (PyObject *)bs;
@@ -245,62 +250,15 @@ static PyObject *pyqtBoundSignal_mp_subscript(PyObject *self,
 {
     qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
 
-    // Make sure the subscript is a tuple.
-    PyObject *args;
+    int idx = qpycore_signal_index((qpycore_pyqtSignal *)bs->unbound_signal,
+            subscript, "a bound signal type argument");
 
-    if (PyTuple_Check(subscript))
-    {
-        args = subscript;
-    }
-    else
-    {
-        args = PyTuple_New(1);
-
-        if (!args)
-            return 0;
-
-        PyTuple_SET_ITEM(args, 0, subscript);
-    }
-
-    Py_INCREF(subscript);
-
-    // Parse the subscript as a tuple of types.
-    Chimera::Signature *ss_signature = Chimera::parse(args, 0,
-            "a bound signal type argument");
-
-    Py_DECREF(args);
-
-    if (!ss_signature)
+    if (idx < 0)
         return 0;
 
-    // Search for an overload with this signature.
-    Chimera::Signature *new_overload = 0;
-    qpycore_pyqtSignal *unbound_signal = (qpycore_pyqtSignal *)bs->unbound_signal;
-
-    for (int i = 0; i < unbound_signal->overloads->size(); ++i)
-    {
-        Chimera::Signature *oload = unbound_signal->overloads->at(i);
-
-        if (oload->arguments() == ss_signature->signature)
-        {
-            new_overload = oload;
-            break;
-        }
-    }
-
-    delete ss_signature;
-
-    if (!new_overload)
-    {
-        PyErr_SetString(PyExc_KeyError,
-                "there is no matching overloaded signal");
-
-        return 0;
-    }
-
     // Create a new bound signal.
-    return qpycore_pyqtBoundSignal_New((PyObject *)unbound_signal,
-            bs->bound_pyobject, bs->bound_qobject, new_overload);
+    return qpycore_pyqtBoundSignal_New(bs->unbound_signal, bs->bound_pyobject,
+            bs->bound_qobject, idx);
 }
 
 
@@ -414,13 +372,20 @@ static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
 static PyObject *connect(qpycore_pyqtBoundSignal *bs, QObject *qrx,
         const char *slot, Qt::ConnectionType type)
 {
-    if (!QObject::connect(bs->bound_qobject, bs->bound_overload->signature.constData(), qrx, slot, type))
+    bool ok;
+
+    Py_BEGIN_ALLOW_THREADS
+    ok = QObject::connect(bs->bound_qobject,
+            bs->bound_overload->signature.constData(), qrx, slot, type);
+    Py_END_ALLOW_THREADS
+
+    if (!ok)
     {
-        QByteArray tx_name = bs->bound_overload->name();
         QByteArray rx_name = Chimera::Signature::name(slot);
 
-        PyErr_Format(PyExc_TypeError, "connect() failed between '%s' and '%s'",
-                tx_name.constData() + 1, rx_name.constData() + 1);
+        PyErr_Format(PyExc_TypeError, "connect() failed between %s and %s()",
+                bs->bound_overload->py_signature.constData(),
+                rx_name.constData() + 1);
 
         return 0;
     }
@@ -508,7 +473,14 @@ static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args)
 static PyObject *disconnect(qpycore_pyqtBoundSignal *bs, QObject *qrx,
         const char *slot)
 {
-    if (!QObject::disconnect(bs->bound_qobject, bs->bound_overload->signature.constData(), qrx, slot))
+    bool ok;
+
+    Py_BEGIN_ALLOW_THREADS
+    ok = QObject::disconnect(bs->bound_qobject,
+            bs->bound_overload->signature.constData(), qrx, slot);
+    Py_END_ALLOW_THREADS
+
+    if (!ok)
     {
         QByteArray tx_name = bs->bound_overload->name();
 
@@ -561,9 +533,11 @@ static PyObject *pyqtBoundSignal_emit(PyObject *self, PyObject *args)
 static QObject *get_receiver(Chimera::Signature *overload, PyObject *slot_obj,
         QByteArray &name)
 {
-    PyObject *rx_self;
+    PyObject *rx_self, *decorations;
     QByteArray rx_name;
-    bool need_qt_slot;
+    bool try_qt_slot;
+
+    decorations = 0;
 
     if (PyMethod_Check(slot_obj))
     {
@@ -580,7 +554,19 @@ static QObject *get_receiver(Chimera::Signature *overload, PyObject *slot_obj,
         Py_DECREF(f_name_obj);
 
         // See if this has been decorated.
-        need_qt_slot = PyObject_HasAttr(f, qpycore_signature_attr_name);
+        decorations = PyObject_GetAttr(f, qpycore_signature_attr_name);
+
+        if (decorations)
+        {
+            try_qt_slot = true;
+
+            // It's convenient to do this here as it's not going to disappear.
+            Py_DECREF(decorations);
+        }
+        else
+        {
+            try_qt_slot = false;
+        }
     }
     else if (PyCFunction_Check(slot_obj))
     {
@@ -593,7 +579,7 @@ static QObject *get_receiver(Chimera::Signature *overload, PyObject *slot_obj,
         if (rx_name.endsWith('_'))
             rx_name.chop(1);
 
-        need_qt_slot = true;
+        try_qt_slot = true;
     }
     else
     {
@@ -614,48 +600,30 @@ static QObject *get_receiver(Chimera::Signature *overload, PyObject *slot_obj,
 
     QObject *rx_qobj = reinterpret_cast<QObject *>(rx);
 
-    if (need_qt_slot)
+    // If there might be a Qt slot that can handle the arguments (or a subset
+    // of them) then use it.  Otherwise we will fallback to using a proxy.
+    if (try_qt_slot)
     {
-        // Get the list of signal arguments.
-        QByteArray args = overload->arguments();
-        args.remove(0, 1);
-        args.chop(1);
-        QList<QByteArray> arg_list = args.split(',');
-
-        // Try to find a Qt slot that can handle the arguments (or a subset of
-        // them).
-        for (int ol = arg_list.count(); ol >= 0; --ol)
+        for (int ol = overload->parsed_arguments.count(); ol >= 0; --ol)
         {
-            // Build the normalised signature that a Qt slot would have.
-            name = rx_name;
-            name.append('(');
+            // If there are decorations then we compare the signal's signature
+            // against them so that we distinguish between Python types that
+            // are passed to Qt as PyQt_PyObject objects.  Qt will not make the
+            // distinction.  If there are no decorations then let Qt determine
+            // if a slot is available.
+            if (decorations)
+                name = slot_signature_from_decorations(overload, decorations,
+                        ol);
+            else
+                name = slot_signature_from_metaobject(overload,
+                        rx_qobj->metaObject(), rx_name, ol);
 
-            for (int a = 0; a < ol; ++a)
-            {
-                if (a != 0)
-                    name.append(',');
-
-                name.append(arg_list[a]);
-            }
-
-            name.append(')');
-
-            if (rx_qobj->metaObject()->indexOfSlot(name.constData()) >= 0)
+            if (!name.isEmpty())
             {
                 // Prepend the magic slot marker.
                 name.prepend('1');
                 break;
             }
-
-            name.clear();
-        }
-
-        if (name.isEmpty())
-        {
-            // There was no appropriate Qt slot.
-            PyErr_Format(PyExc_TypeError,
-                    "'%s()' has no overload that is compatible with '%s'",
-                    rx_name.constData(), overload->signature.constData() + 1);
         }
     }
 
@@ -663,6 +631,113 @@ static QObject *get_receiver(Chimera::Signature *overload, PyObject *slot_obj,
 }
 
 
+// Return the full name and signature of a Qt slot that a signal can be
+// connected to, taking the slot decorators into account.
+static QByteArray slot_signature_from_decorations(Chimera::Signature *signal,
+        PyObject *decorations, int nr_args)
+{
+    for (SIP_SSIZE_T i = 0; i < PyList_GET_SIZE(decorations); ++i)
+    {
+        Chimera::Signature *slot = Chimera::Signature::fromPyObject(
+                PyList_GET_ITEM(decorations, i));
+
+        if (slot->parsed_arguments.count() != nr_args)
+            continue;
+
+        int a;
+
+        for (a = 0; a < nr_args; ++a)
+        {
+            const Chimera *sig_arg = signal->parsed_arguments.at(a);
+            const Chimera *slot_arg = slot->parsed_arguments.at(a);
+
+            // The same type names must be compatible.
+            if (sig_arg->name() == slot_arg->name())
+                continue;
+
+            enum Match {
+                // The type is PyQt_PyObject because it was explicitly
+                // specified as such as a string.
+                MatchesAll,
+
+                // The type is PyQt_PyObject because it was specified as a type
+                // object that needed wrapping.
+                MatchesPyType,
+
+                // The type is something other than PyQt_PyObject.
+                MatchesName
+            };
+
+            Match sig_match, slot_match;
+
+            if (sig_arg->name() != "PyQt_PyObject")
+                sig_match = MatchesName;
+            else
+                sig_match = sig_arg->py_type() ? MatchesPyType : MatchesAll;
+
+            if (slot_arg->name() != "PyQt_PyObject")
+                slot_match = MatchesName;
+            else
+                slot_match = slot_arg->py_type() ? MatchesPyType : MatchesAll;
+
+            // They are incompatible unless one is called PyQt_PyObject.
+            if (sig_match == MatchesName || slot_match == MatchesName)
+                break;
+
+            // They are compatible if neither was a Python type.
+            if (sig_match == MatchesAll || slot_match == MatchesAll)
+                continue;
+
+            // The signal type can be a sub-type of the slot type.
+            if (!PyType_IsSubtype((PyTypeObject *)sig_arg->py_type(), (PyTypeObject *)slot_arg->py_type()))
+                break;
+        }
+
+        if (a == nr_args)
+            return slot_signature(signal, slot->name(), nr_args);
+    }
+
+    return QByteArray();
+}
+
+
+// Return the full name and signature of a Qt slot that a signal can be
+// connected to, taking the Qt meta-object into account.
+static QByteArray slot_signature_from_metaobject(Chimera::Signature *signal,
+        const QMetaObject *slot_mo, const QByteArray &slot_name, int nr_args)
+{
+    QByteArray slot_sig = slot_signature(signal, slot_name, nr_args);
+
+    if (slot_mo->indexOfSlot(slot_sig.constData()) < 0)
+        slot_sig.clear();
+
+    return slot_sig;
+}
+
+
+// Return the full name and signature of the Qt slot that a signal would be
+// connected to.
+static QByteArray slot_signature(Chimera::Signature *signal,
+        const QByteArray &slot_name, int nr_args)
+{
+    QByteArray slot_sig = slot_name;
+
+    slot_sig.append('(');
+
+    for (int a = 0; a < nr_args; ++a)
+    {
+        if (a != 0)
+            slot_sig.append(',');
+
+        slot_sig.append(signal->parsed_arguments.at(a)->name());
+    }
+
+    slot_sig.append(')');
+
+    return slot_sig;
+}
+
+
 // Check that an object is a bound signal and return the bound QObject and the
 // signal signature.
 bool qpycore_pyqtsignal_get_parts(PyObject *sig_obj, QObject **qtx,
